//***************************************************************************************************************************
//*
//*		SCOMM - Command Processing
//*
//* 2014 Oct   : V1 / bhilpert
//*
//***************************************************************************************************************************
#include <unistd.h>
#include <stdio.h>
#include "util.h"
#include "scp.h"

extern int			Sndr_run;
extern int			Sndr_doLine;
extern int			Sndr_doEcho;
extern int			Sndr_knl;
extern int			Sndr_fnl;

extern void			Sndr_wByte();


//----------------------------------------------
// Commander config, assigned during main init

int				Cmdr_wPipe;


//----------------------------------------------
// In-program help comments

private char			*(HelpSummary[]) = {
	"sl*ine [off]			: Set sender mode to line-at-a-time or character-immediate.",
	"se*cho [off]			: Enable/disable sender character-immediate echoing.",
	"sknl [off|crlf|cr]		: Enable/disable sender keyboard NL->CRLF translation.",
	"sfnl [off|crlf|cr]		: Enable/disable sender file NL->CRLF translation.",
	"sf*ile <file>			: Send the specified file to the port.",
	"sby*tes [<num>]*		: Send each numeric value as a byte.",
	"srep*eat <string> [<num>]	: Send <string> repeatedly, <num> times.",
	"sbr*eak <duration>		: Send break.",
	"test*path [clean]		: Send 256 bytes, from 0 to 255, to the port.",
	"",
	"rdhex [off|all]		: Control non-text hex display, off=binary (transparent), on=undefs in hex, all=undefs & ctls in hex.",
	"rdnl [off]			: Turn on/off receiver display CRLF->NL translation mode.",
	"rfnl [off]			: Turn on/off receiver file CRLF->NL translation mode.",
	"rf*ile [[+s] <file>]		: Collect received bytes in file, terminate collecting if no args.",
	"",
	"co*de [ita*2]			: Set character code translation",
	"s*tatus			: Display current status & mode info.",
	"h*elp | ?			: Display command summary.",
	"q*uit | exit			: Terminate SCOMM.",
	NULL };


//----------------------------------------------
// Send directive to receiver through pipe

private void
NotifyReceiver( cmd, arg )
	char			*cmd, *arg;
 {
	char			cmdBuf[100];

	sprintf( cmdBuf, "\x01%s%s", cmd, arg?arg:"" );				// compose directive

	write( Cmdr_wPipe, cmdBuf, 100 );					// and write it to receiver

	sleepuS( 1000 );							// HACK: allow receiver to run
 }


//* END Private
//***************************************************************************************************************************
//* Public


void
Commander( cmdLine, commPort )
	char			*cmdLine;
	SCP			*commPort;
 {
	char			*cmd, *arg, *str;
	FILE			*sendFd;
	int			i, j;

	cmd = s_token( &cmdLine );						// parse first token as command
	if( !cmd || !*cmd )
		return;

	//----------------------------------------------------------------------------------------------------------
	// Sender commands

	if( s_kwMatch("sl*ine",cmd) )						// COMMAND: sender line mode
	 {	arg = s_token( &cmdLine );
		Sndr_doLine = !s_kwMatch("off",arg);

		return;
	 }

	//---------------------------------------

	if( s_kwMatch("se*cho",cmd) )						// COMMAND: secho
	 {	arg = s_token( &cmdLine );
		Sndr_doEcho = !s_kwMatch("off",arg);

		return;
	 }

	//---------------------------------------

	if( s_kwMatch("sknl",cmd) )						// COMMAND: sknl
	 {	arg = s_token( &cmdLine );
		Sndr_knl = s_kwMatch("off",arg) ? 0 : (s_kwMatch("cr",arg) ? 1 : 2);

		return;
	 }

	//---------------------------------------

	if( s_kwMatch("sfnl",cmd) )						// COMMAND: sfnl
	 {	arg = s_token( &cmdLine );
		Sndr_fnl = s_kwMatch("off",arg) ? 0 : (s_kwMatch("cr",arg) ? 1 : 2);

		return;
	 }

	//---------------------------------------

	if( s_kwMatch("sf*ile",cmd) )						// COMMAND: send file
	 {	arg = s_token( &cmdLine );
		if( !arg )
			goto useErr;

		sendFd = fopen( arg, "r" );
		if( !sendFd )
		 {	fprintf( stderr, "[SCOMM: error opening send file %s]\n", arg );
			return;
		 }
		fprintf( stderr, "[SCOMM: sending file %s]\n", arg );

		while( (i=fgetc(sendFd)) != EOF )				// transfer bytes
			Sndr_wByte( commPort, i, Sndr_fnl );

		fclose( sendFd );
		//tcdrain( commPort->fno );					// !!! not working !!!
		fprintf( stderr, "[SCOMM: sending complete]\n" );

		return;
	 }

	//---------------------------------------
	/*
	if( s_kwMatch("canc*el",cmd) )						// COMMAND: cancel sending
	 {	tcflush( commPort->fno, TCOFLUSH );				// !!! not working !!!

		return;
	 }
	*/
	//---------------------------------------

	if( s_kwMatch("sby*tes",cmd) )						// COMMAND: send bytes
	 {	while( (arg=s_token(&cmdLine)) )
		 {	if( sscanf(arg,"%i",&i) != 1 )
				goto useErr;
			scp_wByte( commPort, i );
		 }

		return;
	 }

	//---------------------------------------

	if( s_kwMatch("srep*eat",cmd) )						// COMMAND: send a string or characters repeatedly
	 {	str = s_token( &cmdLine );
		if( !str )
			goto useErr;

		arg = s_token( &cmdLine );
		if( !arg )
			i = 1;
		else if( sscanf(arg,"%i",&i) != 1 )
			goto useErr;

		while( i-- )
			for( j=0; str[j]; ++j )
				scp_wByte( commPort, str[j] );

		return;
	 }

	//---------------------------------------

	if( s_kwMatch("sbr*eak",cmd) )						// COMMAND: send break
	 {	arg = s_token( &cmdLine );
		if( !arg )
			i = 0;
		else
			if( sscanf(arg,"%i",&i) != 1 )
				goto useErr;

		scp_break( commPort, i );

		return;
	 }

	//---------------------------------------

	if( s_kwMatch("test*path",cmd) )					// COMMAND: test path
	 {	arg = s_token( &cmdLine );

		for( i=0; i<256; ++i )
		 {	scp_wByte( commPort, i );
			if( !arg && (i&0xF)==15 ) Sndr_wByte( commPort, '\r', Sndr_knl );
		 }
		return;
	 }

	//----------------------------------------------------------------------------------------------------------
	// Receiver commands

	if( s_kwMatch("rdhex",cmd) )						// COMMAND: rdhex
	 {	arg = s_token( &cmdLine );
		NotifyReceiver( "hd", s_kwMatch("off",arg) ? "0" : (s_kwMatch("all",arg)?"2":"1") );

		return;
	 }

	//---------------------------------------

	if( s_kwMatch("rdnl",cmd) )						// COMMAND: rdnl
	 {	arg = s_token( &cmdLine );
		NotifyReceiver( "ld", s_kwMatch("off",arg) ? "0" : "1" );

		return;
	 }

	//---------------------------------------

	if( s_kwMatch("rfnl",cmd) )						// COMMAND: rfnl
	 {	arg = s_token( &cmdLine );
		NotifyReceiver( "lf", s_kwMatch("off",arg) ? "0" : "1" );

		return;
	 }

	//---------------------------------------

	if( s_kwMatch("rf*ile",cmd) )						// COMMAND: collect to file
	 {	arg = s_token( &cmdLine );
		if( arg )							// open file for collecting
		 {	if( s_kwMatch("+s",arg) )				// check for suppress-display option
			 {	if( !(arg=s_token(&cmdLine)) )
					goto useErr;
				NotifyReceiver( "fs", arg );
			 }
			else
				NotifyReceiver( "f-", arg );
		 }
		else
			NotifyReceiver( "fe", NULL );				// terminate collecting

		return;
	 }

	//----------------------------------------------------------------------------------------------------------
	// Miscellaneous commands

	if( s_kwMatch("co*de",cmd) )						// COMMAND: code
	 {	arg = s_token( &cmdLine );

		scp_code( commPort, arg?arg:"" );				// set sender translation
		NotifyReceiver( "c", arg );					// tell receiver to translate too

		return;
	 }

	//---------------------------------------

	if( s_kwMatch("s*tatus",cmd) )						// COMMAND: status
	 {	fprintf( stderr, "[SCOMM: mode=%s, echo=%s, sknl=%s, sfnl=%s, code=%s]\n",	// display sender status
			Sndr_doLine	? "line-at-a-time" : "char-immediate",
			Sndr_doEcho	? "on" : "off",
			Sndr_knl==0	? "off" : (Sndr_knl==1?"cr":"crlf"),
			Sndr_fnl==0	? "off" : (Sndr_fnl==1?"cr":"crlf"),
			scp_code(commPort,NULL) );

		NotifyReceiver( "s", NULL );					// tell receiver to display status too

		return;
	 }

	//---------------------------------------

	if( s_kwMatch("h*elp",cmd) || s_kwMatch("?",cmd) )			// COMMAND: help
	 {	for( i=0; (arg=HelpSummary[i]); ++i )
			fprintf( stderr, "[SCOMM:   %s]\n", arg );

		return;
	 }

	//---------------------------------------

	if( s_kwMatch("q*uit",cmd) || s_kwMatch("exit",cmd) )			// COMMAND: quit
	 {	Sndr_run = FALSE;

		return;
	 }

	//---------------------------------------

 useErr:									// bad command
	fprintf( stderr, "[SCOMM: bad command (%s)]\n", cmd );
 }


//** EOF
//***************************************************************************************************************************
