//***************************************************************************************************************************
//*
//*			SCOMM - Loopback Serial Communication Port Interface
//*
//* 2014 Nov   : V1 / bhilpert
//*
//*
//*	BUG: fsyncs() added on write pipe in hopes of flushing at termination, but only partially works.
//*	Sender should wait for receiver to terminate.
//*
//***************************************************************************************************************************
#include <unistd.h>
#include <stdio.h>
#include <errno.h>
#include "util.h"


//----------------------------------------------
// Interface config & state descriptor

typedef struct
 {	int			rfno;
	int			wfno;
 }
	CPLPB;

private CPLPB			_lpb;						// HACK: global


//* END Private
//***************************************************************************************************************************
//* Public

//----------------------------------------------
// Open loopback comm port

void*
cpLPB_open( devName, speed, bits )
	char*			devName;
	int			speed, bits;
 {
	int			loopbackPipe[2];

	//--------------------------------
	// Create loopback pipe

	if( pipe(loopbackPipe) )						// prepare a pipe for loopback from sender to receiver
	 {	fprintf( stderr, "  -- error opening loopback pipe\n" );
		return( NULL );
	 }

	//--------------------------------
	// Init descriptor

	_lpb.rfno = loopbackPipe[0];						// receiver shall use read end of pipe
	_lpb.wfno = loopbackPipe[1];						// sender shall use write end of pipe

	return( &_lpb );
 }


//----------------------------------------------
// Close interface

void
cpLPB_close( lpb )
	CPLPB*			lpb;
 {
	fsync( lpb->wfno );

	close( lpb->rfno );
	close( lpb->wfno );
 }


//----------------------------------------------
// Read a byte

int
cpLPB_rByte( lpb )
	CPLPB*			lpb;
 {
	char			b[1];

	if( read(lpb->rfno,b,1) <= 0 )
		return( errno==EINTR ? -2 : -1 );

	return( b[0] & 0xFF );
 }


//----------------------------------------------
// Write a byte

void
cpLPB_wByte( lpb, byte )
	CPLPB*			lpb;
	int			byte;
 {
	char			b[1];

	b[0] = byte;
	write( lpb->wfno, b, 1 );
 }


//** EOF
//***************************************************************************************************************************
