//***************************************************************************************************************************
//*
//*			SCOMM - ITA2 5-bit Character Code translation
//*
//* 2014 Oct   : V1 / bhilpert
//*
//***************************************************************************************************************************
#include "util.h"


//-------------------------------------------------------------
// ITA2 code definitions and tables

//#define ITA2_NULL		0
//#define ITA2_LF		2
//#define ITA2_CR		8
//#define ITA2_BELL		5					// figs only
#define ITA2_FIGS		27
#define ITA2_LTRS		31

private char			ITA2_LtrsToAscTbl[] = "\0" "E\nA " "S"    "IU\rDRJNFCKT" "Z"  "LWHYPQOBG~MXV~";
private char			ITA2_FigsToAscTbl[] = "\0" "3\n- " "\x07" "87\r$4',!:(5" "\"" ")2#6019?&~./;~";
private int			ITA2_FromAscTbl[256];


//-------------------------------------------------------------
// Interface config & state

typedef struct
 {	void			*cpxxx;					// descriptor to lower level R/W
	int			(*cprByte)();				// lower level read
	void			(*cpwByte)();				// lower level write

	int			wShift;					// write shift state, this is LTRS or FIGS code
	char*			rShift;					// read shift state, this is Ltrs or Figs table
 }
	ITA2;

private ITA2			_ita2;					// HACK: global


//* END Private
//***************************************************************************************************************************
//* Public

void*
ita2_insert( cpxxx, rByteF, wByteF )
	void			*cpxxx;
	int			(*rByteF)();
	void			(*wByteF)();
 {
	int			i, ascLtr, ascFig;

	//--------------------------------
	// Init translation table

	for( i=0; i<256; ++i )						// clear table to invalids (no equivalent for ASCII character in ITA2)
		ITA2_FromAscTbl[i] = -1;

	for( i=0; i<32; ++i )						// build ASCII-->ITA2 code table from ITA2-->ASCII tables
	 {	ascLtr = (int) ITA2_LtrsToAscTbl[i];
		ascFig = (int) ITA2_FigsToAscTbl[i];
		if( ascLtr == '~' )
			continue;
		if( ascLtr == ascFig )
			ITA2_FromAscTbl[ ascLtr ] = i | 0;		// character is good in both shift states
		else
		 {	ITA2_FromAscTbl[ ascLtr ] = i | (ITA2_LTRS<<8);	// required shift state is encoded in higher byte
			ITA2_FromAscTbl[ ascFig ] = i | (ITA2_FIGS<<8);
		 }
	 }

	for( i='a'; i<='z'; ++i )					// translate lower case to upper
		ITA2_FromAscTbl[i] = ITA2_FromAscTbl[i-32];

	//--------------------------------
	// Config

	_ita2.cpxxx	= cpxxx;					// save lower-level comm-port desc & r/w functions
	_ita2.cprByte	= rByteF;
	_ita2.cpwByte	= wByteF;

	//--------------------------------
	// Init state

	_ita2.wShift = 0;						// shift state of tty printer is initially unknown,
									//	force an initial shift to desired state
	_ita2.rShift = ITA2_LtrsToAscTbl;				// start with letters state table for receive

	return( &_ita2 );
 }


//-------------------------------------------------------------
// Remove translation

void*
ita2_remove( ita2, ret_rByteF, ret_wByteF )
	ITA2			*ita2;
	int			(**ret_rByteF)();
	void			(**ret_wByteF)();
 {
	*ret_rByteF	= ita2->cprByte;				// return original config
	*ret_wByteF	= ita2->cpwByte;

	return( ita2->cpxxx );
 }


//-------------------------------------------------------------
// Read char and translate ITA2 to ASCII

int
ita2_rByte( ita2 )
	ITA2			*ita2;
 {
	int			c;

	repeat
	 {	c = (*ita2->cprByte)( ita2->cpxxx );
		if( c < 0 )
			return( c );

		if( c == ITA2_LTRS )
			ita2->rShift = ITA2_LtrsToAscTbl;		// shift state to letters
		else if( c == ITA2_FIGS )
			ita2->rShift = ITA2_FigsToAscTbl;		// shift state to characters
		else if( c < 32 )
			return( ita2->rShift[c] );			// an actual character
		else
			return( 0x7F );					// shouldn't normally happen
	 }
 }


//-------------------------------------------------------------
// Translate ASCII byte to ITA2 and send

void
ita2_wByte( ita2, byte )
	ITA2*			ita2;
	int			byte;
 {
	int			itaCode, shiftCode;

	itaCode = ITA2_FromAscTbl[byte];				// translate ASCII to ITA2
	if( itaCode < 0 ) itaCode = ITA2_FromAscTbl['?'];		// no ITA2 character

	shiftCode = itaCode >> 8;

	if( shiftCode && shiftCode!=ita2->wShift )			// change shift state if necessary
	 {	ita2->wShift = shiftCode;
		(*ita2->cpwByte)( ita2->cpxxx, ita2->wShift );
	 }

	(*ita2->cpwByte)( ita2->cpxxx, itaCode&0xFF );			// finally send	translated character
 }


//** EOF
//***************************************************************************************************************************
