//***************************************************************************************************************************
//*
//*			SCOMM
//*		Serial Communication Program
//*			Main
//*
//* 2014 Oct   : V1 / bhilpert
//*
//***************************************************************************************************************************
#include <stdlib.h>
#include <unistd.h>
#include <termios.h>
#include <stdio.h>
#include <string.h>
#include "util.h"
#include "scp.h"

extern void			Sender();
extern void			Receiver();
extern void			Commander();
extern int			Cmdr_wPipe;


//------------------------------------------------------------------
// Concatenate an arg sequence of the form -<arg> <arg>* into one string.
// Return the new argi (index of next arg to process).
// BUG: not checking for cmd string overflow.

private int
argcat( cmd, argi, argc, argv )
	char			*cmd;
	int			argi, argc;
	char			**argv;
 {
	strcpy( cmd, &argv[argi][1] );				// skip '-' at beginning of first arg

	while( ++argi<argc && argv[argi][0]!='-' )		// stop when next arg begins with '-'
	 {	strcat( cmd, " " );
		strcat( cmd, argv[argi] );
	 }

	return( argi - 1 );
 }


//------------------------------------------------------------------
// User TTY Config

private int			tty__fno;
private struct termios		tty__restoreAttrs;


private void
tty__restore( void )
 {
	tcsetattr( tty__fno, TCSANOW, &tty__restoreAttrs );
 }


private void
Tty_cfg( fno )
	int			fno;
 {
	struct termios		tattr;

	if( !isatty(fno) )
	 {	fprintf( stderr, "  -- stdio is not a tty\n" );
		return;
	 }

	tcgetattr( fno, &tty__restoreAttrs );					// save mode for restoration on exit
	tty__fno = fno;
	atexit( tty__restore );

	tcgetattr( fno, &tattr );						// get attributes to modify

	tattr.c_lflag &= ~(ICANON|ECHO);					// turn off input canonical processing and echo
	tattr.c_cc[VMIN]  = 1;
	tattr.c_cc[VTIME] = 0;

	tattr.c_iflag &= ~(IGNCR|ICRNL|INLCR);					// no translation of CR & NL

	if( tcsetattr(fno,TCSAFLUSH,&tattr) < 0 )				// set new attributes
	 {	fprintf( stderr, "  -- error setting stdio tty attributes\n" );
		return;
	 }
 }


//* END Private
//***************************************************************************************************************************
//* Main

int
main( argc, argv )
	int			argc;
	char**			argv;
 {
	char*			commDev;
	int			speed, bits, cmdMode;
	SCP*			commPort;
	char			cmdBuf[200];
	int			srPipe[2];					// pipe for sender->receiver directives
	pid_t			rcvrPid;
	int			i;

	//---------------------------------------
	// Configuration

	commDev		= "/dev/tty.usbserial";
	speed		= 9600;
	bits		= 8;
	cmdMode		= FALSE;

	for( i=1; i<argc; ++i )
		if( strcmp(argv[i],"-d") == 0 )
		 {	argv[i] = NULL;						// mark arg as processed
			if( ++i == argc ) goto useFail;
			commDev = argv[i];
			argv[i] = NULL;
		 }
		else if( strcmp(argv[i],"-s") == 0 )
		 {	argv[i] = NULL;
			if( ++i == argc ) goto useFail;
			if( sscanf(argv[i],"%d",&speed) != 1 ) goto useFail;
			argv[i] = NULL;
		 }
		else if( strcmp(argv[i],"-b") == 0 )
		 {	argv[i] = NULL;
			if( ++i == argc ) goto useFail;
			if( sscanf(argv[i],"%d",&bits) != 1 ) goto useFail;
			argv[i] = NULL;
		 }
		else if( strcmp(argv[i],"-c") == 0 )
		 {	cmdMode = TRUE;
			argv[i] = NULL;
		 }
		else if( strcmp(argv[i],"-u") == 0 )
			goto useFail;
		else
			;							// leave arg for command exec

	if( !commDev )
	 {useFail:
		fprintf( stderr, "  -- Use: %s  [-u]  [-d <commDevice>]  [-s <speed>]  [-b <bits>]  [-c]  [-<cmd> <args>*]*\n", argv[0] );
		return( EXIT_FAILURE );
	 }

	//---------------------------------------
	// Initialisation

	Tty_cfg( 0 );								// config keyboard/display (stdin/stdout)

	commPort = scp_open( commDev, speed, bits );				// open the communication port
	if( !commPort )
		return( EXIT_FAILURE );

	if( pipe(srPipe) )							// prepare a pipe for commands to the receiver
	 {	fprintf( stderr, "  -- error opening pipe\n" );
		return( EXIT_FAILURE );
	 }

	rcvrPid = fork();							// split receiver and sender processes
	if( rcvrPid < 0 )
	 {	fprintf( stderr, "  -- error on fork\n" );
		return( EXIT_FAILURE );
	 }

	//---------------------------------------
	// Receiver & Sender Activity

	if( rcvrPid == 0 )							// RECEIVER
	 {	write( srPipe[1], "r", 1 );					// ack to sender
		close( srPipe[1] );						// receiver/child closes write end of pipe

		Receiver( commPort, srPipe[0] );

		close( srPipe[0] );						// close read end of pipe
	 }
	else									// SENDER
	 {	read( srPipe[0], cmdBuf, 1 );					// wait for receiver to run
		close( srPipe[0] );						// sender/parent closes read end of pipe

		Cmdr_wPipe = srPipe[1];

		for( i=1; i<argc; ++i )						// execute commands from command-line
		 {	if( !argv[i] || argv[i][0]!='-' )			// skip already-processed args
				continue;
			i = argcat( cmdBuf, i, argc, argv );			// concat args into command

			Commander( cmdBuf, commPort );				// execute args as command
		 }

		Sender( commPort, cmdMode );

		close( srPipe[1] );						// close write end of pipe, this terminates receiver
		waitpid( rcvrPid, NULL, 0 );					// wait for receiver to complete
	 }

	//---------------------------------------
	// Termination

	scp_close( commPort );

	return( EXIT_SUCCESS );
 }


//** EOF
//***************************************************************************************************************************
