//***************************************************************************************************************************
//*
//*			SCOMM - Serial Communication Port Interface
//*
//* 2014 Oct   : V1 / bhilpert
//*
//*	Unified interface for the comm port. Provides for redirection to different hardware interfaces, and code translation.
//*
//***************************************************************************************************************************
#include <string.h>
#include "util.h"
#include "scp.h"

extern void*			cpTTY_open();
extern void			cpTTY_close();
extern int			cpTTY_rByte();
extern void			cpTTY_wByte();

extern void*			cpSBB_open();
extern void			cpSBB_close();
extern int			cpSBB_rByte();
extern void			cpSBB_wByte();

extern void*			cpLPB_open();
extern void			cpLPB_close();
extern int			cpLPB_rByte();
extern void			cpLPB_wByte();

extern void*			ita2_insert();
extern void*			ita2_remove();
extern int			ita2_rByte();
extern void			ita2_wByte();


//----------------------------------------------
// Config descriptor

private SCP			_scp;					// HACK: global


//* END Private
//***************************************************************************************************************************
//* Public

//----------------------------------------------
// Open interface

SCP*
scp_open( devName, speed, bits )
	char			*devName;
	int			speed, bits;
 {
	SCP			*scp = &_scp;

	if( s_eql(devName,"sbb") )
	 {	scp->cpxxx	= cpSBB_open( devName, speed, bits );
		scp->closeF	= &cpSBB_close;
		scp->rByteF	= &cpSBB_rByte;
		scp->wByteF	= &cpSBB_wByte;
	 }
	else if( s_eql(devName,"loopback") )
	 {	scp->cpxxx	= cpLPB_open( devName, speed, bits );
		scp->closeF	= &cpLPB_close;
		scp->rByteF	= &cpLPB_rByte;
		scp->wByteF	= &cpLPB_wByte;
	 }
	else
	 {	scp->cpxxx	= cpTTY_open( devName, speed, bits );
		scp->closeF	= &cpTTY_close;
		scp->rByteF	= &cpTTY_rByte;
		scp->wByteF	= &cpTTY_wByte;
	 }
	if( !scp->cpxxx )
		return( NULL );

	scp->codeName = NULL;						// default (no translation) character code is ASCII

	return( scp );
 }


//----------------------------------------------
// Close interface

void
scp_close( scp )
	SCP			*scp;
 {
	(*scp->closeF)( scp->cpxxx );
 }


//----------------------------------------------
// Set character code translation

char*
scp_code( scp, codeName )
	SCP			*scp;
	char			*codeName;
 {
	if( codeName )							// just return active code if no name specified
	 {	if( scp->codeName )					// remove current translation if active
		 {	scp->cpxxx = (*scp->codeRemoveF)( scp->cpxxx, &scp->rByteF, &scp->wByteF );
			scp->codeName = NULL;
		 }

		if( s_kwMatch("ita*2",codeName) )			// insert new translation if specified
		 {	scp->cpxxx = ita2_insert( scp->cpxxx, scp->rByteF, scp->wByteF );

			scp->codeName		= "ITA2";
			scp->codeRemoveF	= &ita2_remove;
			scp->rByteF		= &ita2_rByte;		// insert ita2 functions
			scp->wByteF		= &ita2_wByte;
		 }
	 }

	return( scp->codeName ? scp->codeName : "ASCII" );
 }


//----------------------------------------------
// Send break

void
scp_break( scp, duration )
	SCP			*scp;
	int			duration;
 {
	// not implemented as tcsendbreak didn't work on USB dev anyways
	//tcsendbreak( commPort->fno, duration );
 }


//** EOF
//***************************************************************************************************************************
