//***************************************************************************************************************************
//*
//*		SCOMM - Sender Process
//*
//* 2014 Oct   : V1 / bhilpert
//*
//***************************************************************************************************************************
#include <stdio.h>
#include "util.h"
#include "scp.h"

extern void			Commander();


//----------------------------------------------

#define LINEBF_SIZE		100

#define ST_SEND			1
#define ST_CMD			2

#define CHR_CMD			'\x01'			// CTL-A
#define CHR_END			'\x04'			// CTL-D


//* END Private
//***************************************************************************************************************************
//* Public

//----------------------------------------------
// Sender state

int				Sndr_run	= TRUE;
int				Sndr_doLine	= FALSE;
int				Sndr_doEcho	= TRUE;
int				Sndr_knl	= FALSE;					// keyboard new-line translation
int				Sndr_fnl	= FALSE;					// send-file new-line translation


//----------------------------------------------
// Send a byte with configured translations

void
Sndr_wByte( port, c, nlTranslation )
	SCP*			port;
	int			c, nlTranslation;
 {
	if( nlTranslation>0 && (c=='\r' || c=='\n') )
	 {	scp_wByte( port, '\r' );
		if( nlTranslation > 1 )
		 {	scp_wByte( port, '\n' );
			scp_wByte( port, '\0' );						// HACK: should make null an option
		 }
	 }
	else
		scp_wByte( port, c );
 }


//----------------------------------------------
// Sender main process

void
Sender( commPort, cmdMode )
	SCP*			commPort;
	int			cmdMode;
 {
	char			cmdStr[LINEBF_SIZE+1], sendStr[LINEBF_SIZE+1];
	int			cmdIdx, sendIdx, c, state, i;

	sendIdx = 0;
	cmdIdx = 0;
	state = cmdMode ? ST_CMD: ST_SEND;

	if( cmdMode )
	 {	fprintf( stdout, "[SCOMM> " );
		fflush( stdout );
	 }

	while( Sndr_run && (c=fgetc(stdin))!=EOF && c!=CHR_END )				// get character from user
	 {	switch( state )
		 {	case ST_SEND:
				if( c == CHR_CMD )						// enter command state
				 {	if( sendIdx > 0 ) fputc( '\n', stdout );
					fprintf( stdout, "[SCOMM> " ); fflush( stdout );

					sendIdx = 0;						// discard any partial send line
					cmdIdx = 0;
					state = ST_CMD;
					continue;
				 }

				if( Sndr_doLine )						// line-at-a-time mode
				 {	if( c == '\r' )						// eol, send line to port
					 {	fputc( '\n', stdout );
						sendStr[sendIdx] = 0;
						for( i=0; i<sendIdx; ++i )
							Sndr_wByte( commPort, sendStr[i], Sndr_knl );
						Sndr_wByte( commPort, '\r', Sndr_knl );
						sendIdx = 0;
					 }
					else if( c=='\x08' || c=='\x7F' )			// delete char
					 {	if( sendIdx > 0 )
						 {	fprintf( stdout, "\x08 \x08" ); fflush( stdout );
							--sendIdx;
						 }
					 }
					else if( sendIdx < LINEBF_SIZE )			// check for overflow
					 {	fputc( c, stdout ); fflush( stdout );		// echo
						sendStr[sendIdx++] = c;				// add character to buffer
					 }
					else
					 {	fputc( '\x07', stdout ); fflush( stdout );	// bell for bad input
					 }
				 }
				else								// character-immediate mode
				 {	if( Sndr_doEcho )
					 {	fputc( c=='\r'?'\n':c, stdout ); fflush( stdout );
					 }
					Sndr_wByte( commPort, c, Sndr_knl );			// send to port
					sendIdx = c=='\r' ? 0 : sendIdx+1;			// just to track beginning of line
				 }
				break;

			case ST_CMD:
				if( c=='\r' || c==CHR_CMD )					// eol, execute command
				 {	fprintf( stdout, "]\n" );
					cmdStr[cmdIdx] = 0;

					Commander( cmdStr, commPort );

					if( c == CHR_CMD ) ungetc( c, stdin );			// allow re-entering command state
					state = ST_SEND;
				 }
				else if( c=='\x08' || c=='\x7F' )				// delete char
				 {	if( cmdIdx > 0 )
					 {	fprintf( stdout, "\x08 \x08" ); fflush( stdout );
						--cmdIdx;
					 }
				 }
				else if( c>=' ' && c<='~' && cmdIdx<LINEBF_SIZE	)		// check for valid and overflow
				 {	fputc( c, stdout ); fflush( stdout );			// echo
					cmdStr[cmdIdx++] = c;					// add character to buffer
				 }
				else
				 {	fputc( '\x07', stdout ); fflush( stdout );		// bell for bad input
				 }
		 }
	 }

	fprintf( stdout, "[SCOMM: terminating]\n" );
 }


//** EOF
//***************************************************************************************************************************
